# src/crimson/file_loader/__init__.py
import os
from typing import List, Callable, Optional
from pathlib import Path
import shutil
from .utils import (
    filter_source,
    transform_path,
)


def collect_files(
    source: str,
    out_dir: str,
    separator: str = "%",
    includes: List[str] = [],
    excludes: List[str] = [],
    path_editor: Optional[Callable[[str], str]] = None,
    overwrite: bool = True,
):
    out_dir_path = Path(out_dir)
    if overwrite and out_dir_path.exists():
        shutil.rmtree(out_dir)
    out_dir_path.mkdir(parents=True, exist_ok=True)

    source_paths = filter_source(source, includes, excludes)

    for src_path in source_paths:

        new_path = transform_path(src_path, separator)
        if path_editor:
            new_path = path_editor(new_path)
        new_path = str(Path(out_dir) / new_path)

        Path(new_path).parent.mkdir(parents=True, exist_ok=True)

        shutil.copy2(src_path, new_path)

    print(f"Files collected from {source} to {out_dir}")


def reconstruct_folder_structure(
    source: str,
    out_dir: str,
    separator: str = "%",
    path_editor: Optional[Callable[[str], str]] = None,
    overwrite: bool = True,
):

    out_dir_path = Path(out_dir)

    if overwrite and out_dir_path.exists():
        shutil.rmtree(out_dir)

    out_dir_path.mkdir(parents=True, exist_ok=True)

    for root, _, files in os.walk(source):
        for file in files:

            src_file_path = os.path.join(root, file)

            if path_editor:
                file = path_editor(file)

            relative_path = file.replace(separator, os.path.sep)
            new_file_path = (out_dir + '/' + relative_path).replace('//', '/')

            os.makedirs(os.path.dirname(new_file_path), exist_ok=True)

            shutil.copy2(src_file_path, new_file_path)

    print(f"Folder structure reconstructed from {source} to {out_dir}")



import pytest
import re
from pathlib import Path
from typing import List
from crimson.file_loader.utils import (
    search,
    filter,
    get_paths,
    filter_source,
    transform_path,
    filter_paths,
)


def test_search():
    assert search("test", "This is a test string")
    assert not search("python", "This is a test string")
    assert search("TEST", "This is a test string", flags=[re.IGNORECASE])


def test_filter():
    paths = ["file1.txt", "file2.py", "file3.jpg"]
    assert filter("\.txt$", paths) == ["file1.txt"]
    assert filter("\.py$", paths, mode="include") == ["file2.py"]
    assert filter("\.py$", paths, mode="exclude") == ["file1.txt", "file3.jpg"]
    paths_in_Path: List[Path] = [Path(path) for path in paths]
    assert filter("\.txt$", paths_in_Path) == ["file1.txt"]


def test_get_paths(tmp_path):
    # 임시 디렉토리 구조 생성
    d = tmp_path / "sub"
    d.mkdir()
    (d / "file1.txt").touch()
    (tmp_path / "file2.py").touch()

    paths = get_paths(str(tmp_path))
    assert len(paths) == 2
    assert str(tmp_path / "sub" / "file1.txt") in paths
    assert str(tmp_path / "file2.py") in paths


def test_filter_source(tmp_path):
    # 임시 디렉토리 구조 생성
    d = tmp_path / "sub"
    d.mkdir()
    (d / "file1.txt").touch()
    (tmp_path / "file2.py").touch()
    (tmp_path / "file3.jpg").touch()

    paths = filter_source(tmp_path, includes=["\.txt$", "\.py$"], excludes=["sub"])
    assert len(paths) == 1
    assert str(tmp_path / "file2.py") in paths


def test_transform_path():
    assert transform_path("/path/to/file.txt") == "%path%to%file.txt"
    assert transform_path("/path/to/file.txt", separator="-") == "-path-to-file.txt"


def test_filter_paths():
    paths = [
        "/home/user/docs/file1.txt",
        "/home/user/docs/file2.pdf",
        "/home/user/images/pic1.jpg",
        "/home/user/images/pic2.png",
        "/home/user/code/script.py",
    ]

    # Test with only includes
    result = filter_paths(paths, includes=[r"\.txt$", r"\.pdf$"])
    assert result == ["/home/user/docs/file1.txt", "/home/user/docs/file2.pdf"]

    # Test with only excludes
    result = filter_paths(paths, excludes=[r"\.jpg$", r"\.png$"])
    assert result == [
        "/home/user/docs/file1.txt",
        "/home/user/docs/file2.pdf",
        "/home/user/code/script.py",
    ]

    # Test with both includes and excludes
    result = filter_paths(
        paths, includes=[r"/docs/", r"/images/"], excludes=[r"\.pdf$"]
    )
    assert result == [
        "/home/user/docs/file1.txt",
        "/home/user/images/pic1.jpg",
        "/home/user/images/pic2.png",
    ]

    # Test with no matches in includes
    result = filter_paths(paths, includes=[r"\.mp3$"])
    assert result == []

    # Test with excludes that match everything
    result = filter_paths(paths, excludes=[r".*"])
    assert result == []

    # Test with empty includes and excludes
    result = filter_paths(paths)
    assert result == paths

    # Test with multiple include patterns
    result = filter_paths(paths, includes=[r"\.txt$", r"\.py$"])
    assert result == ["/home/user/docs/file1.txt", "/home/user/code/script.py"]

    # Test with overlapping includes and excludes
    result = filter_paths(
        paths, includes=[r"/docs/", r"/images/"], excludes=[r"\.jpg$", r"/docs/"]
    )
    assert result == ["/home/user/images/pic2.png"]



# src/crimson/file_loader/__init__.py
import os
from typing import List, Literal, Optional, overload, Union
from pathlib import Path
import re


def search(pattern: str, text: str, flags: Optional[List[re.RegexFlag]] = None):
    combined_flags = 0
    if flags:
        for flag in flags:
            combined_flags |= flag

    compiled_pattern = re.compile(pattern, flags=combined_flags)

    is_included = compiled_pattern.search(text) is not None

    return is_included


@overload
def filter(
    pattern: str,
    paths: List[str],
    mode: Literal["include", "exclude"] = "include",
    flags: Optional[List[re.RegexFlag]] = None,
) -> List[str]:
    """doc1"""
    ...


@overload
def filter(
    pattern: str,
    paths: List[Path],
    mode: Literal["include", "exclude"] = "include",
    flags: Optional[List[re.RegexFlag]] = None,
) -> List[str]:
    """doc2"""
    ...


def filter(
    pattern: str,
    paths: Union[List[str], List[Path]],
    mode: Literal["include", "exclude"] = "include",
    flags: Optional[List[re.RegexFlag]] = None,
) -> List[str]:

    if isinstance(paths[0], Path):
        paths = _convert_paths_to_texts(paths)
    paths = _filter_base(
        pattern,
        paths,
        mode,
        flags,
    )

    return paths


def _filter_base(
    pattern: str,
    paths: List[str],
    mode: Literal["include", "exclude"] = "include",
    flags: Optional[List[re.RegexFlag]] = None,
) -> List[str]:
    included = []
    for path in paths:
        is_included = search(pattern, path, flags)
        if mode == "exclude":
            is_included = not is_included
        if is_included:
            included.append(path)
    return included


def _convert_paths_to_texts(paths: List[Path]) -> List[str]:
    texts = [str(path) for path in paths]
    return texts


def get_paths(
    source: str,
):
    paths = []
    for root, _, files in os.walk(source):
        for file in files:
            file_path = Path(root) / file
            paths.append(str(file_path))
    return paths


def filter_paths(
    paths: List[str],
    includes: List[str] = [],
    excludes: List[str] = [],
):

    if len(includes) != 0:
        included_paths = []
        for pattern in includes:
            included_paths += filter(pattern, paths, mode="include")
        paths = included_paths

    for pattern in excludes:
        if len(paths) != 0:
            paths = filter(pattern, paths, mode="exclude")

    return paths


def filter_source(
    source: str,
    includes: List[str] = [],
    excludes: List[str] = [],
):
    paths = get_paths(source)
    paths = filter_paths(paths, includes, excludes)
    return paths


def transform_path(path: str, separator: str = "%") -> str:
    path = path.replace("/", separator)
    return path



import pytest
from crimson.file_loader import collect_files, reconstruct_folder_structure


@pytest.fixture
def sample_directory(tmp_path):
    # 샘플 디렉토리 구조 생성
    (tmp_path / "dir1").mkdir()
    (tmp_path / "dir1" / "file1.txt").write_text("content1")
    (tmp_path / "dir2").mkdir()
    (tmp_path / "dir2" / "file2.py").write_text("content2")
    (tmp_path / "file3.jpg").write_text("content3")
    return tmp_path


@pytest.fixture
def sample_out_directory(tmp_path):
    # 샘플 디렉토리 구조 생성
    (tmp_path / "dir1%file1.txt").write_text("content1")
    (tmp_path / "dir2%file2.py").write_text("content2")
    (tmp_path / "file3.jpg").write_text("content3")
    return tmp_path


def get_new_path(tmp_path, relative_path="dir1/file1.txt"):
    new_path = str(tmp_path / relative_path).replace("/", "%")
    return new_path


def test_collect_files(sample_directory, tmp_path):
    out_dir = tmp_path / "output"
    collect_files(
        str(sample_directory),
        str(out_dir),
        includes=["\.txt$", "\.py$"],
        excludes=["dir2"],
    )

    assert (out_dir / get_new_path(tmp_path, "dir1/file1.txt")).exists()
    assert not (out_dir / get_new_path(tmp_path, "dir2%file2.py")).exists()
    assert not (out_dir / get_new_path(tmp_path, "file3.jpg")).exists()


def test_collect_files_with_path_editor(sample_directory, tmp_path):
    out_dir = tmp_path / "output"

    def path_editor(path):
        return path.replace("dir1", "new_dir1")

    collect_files(
        str(sample_directory),
        str(out_dir),
        path_editor=path_editor,
    )

    assert (out_dir / get_new_path(tmp_path, "new_dir1%file1.txt")).exists()


def test_collect_files_overwrite(sample_directory, tmp_path):
    out_dir = tmp_path / "output"
    out_dir.mkdir()
    (out_dir / "existing_file.txt").write_text("old content")

    assert (out_dir / "existing_file.txt").exists()

    collect_files(str(sample_directory), str(out_dir), overwrite=True)

    assert not (out_dir / "existing_file.txt").exists()


def test_reconstruct_folder_structure(sample_directory, tmp_path):
    intermediate_dir = tmp_path / "intermediate"
    collect_files(str(sample_directory), str(intermediate_dir))

    out_dir = tmp_path / "output"
    reconstruct_folder_structure(str(intermediate_dir), str(out_dir))

    assert (out_dir / tmp_path / "dir1" / "file1.txt").exists()
    assert (out_dir / tmp_path / "dir2" / "file2.py").exists()
    assert (out_dir / tmp_path / "file3.jpg").exists()


def test_reconstruct_folder_structure_with_path_editor(sample_directory, tmp_path):
    intermediate_dir = tmp_path / "intermediate"
    collect_files(str(sample_directory), str(intermediate_dir))

    out_dir = tmp_path / "output"

    def path_editor(path):
        return path.replace(str(tmp_path).replace("/", "%"), "")

    reconstruct_folder_structure(
        str(intermediate_dir), str(out_dir), path_editor=path_editor
    )

    assert (out_dir / "dir1" / "file1.txt").exists()


def test_reconstruct_folder_structure_overwrite(sample_directory, tmp_path):
    intermediate_dir = tmp_path / "intermediate"
    collect_files(str(sample_directory), str(intermediate_dir))

    out_dir = tmp_path / "output"
    out_dir.mkdir()
    (out_dir / "existing_file.txt").write_text("old content")

    assert (out_dir / "existing_file.txt").exists()

    reconstruct_folder_structure(str(intermediate_dir), str(out_dir), overwrite=True)

    assert not (out_dir / "existing_file.txt").exists()
